@@ -4,13 +4,13 @@ import json
 import logging
 import pickle
 
-from tqdm import tqdm
-import numpy as np
-from scipy.stats import bootstrap
+# from tqdm import tqdm
+# import numpy as np
+# from scipy.stats import bootstrap
 
 from allennlp.modules import ConditionalRandomField
 from allennlp.modules.conditional_random_field import allowed_transitions
-from seqeval.metrics import f1_score, precision_score, recall_score, classification_report
+# from seqeval.metrics import f1_score, precision_score, recall_score, classification_report
 
 import os
 import re
@@ -29,6 +29,11 @@ __all__ = 'NER'
 PAD_TOKEN_LABEL_ID = nn.CrossEntropyLoss().ignore_index
 
 
+def user_name_handler(_string):
+    pass
+
+
+
 def pickle_save(obj, path: str):
     with open(path, "wb") as fp:
         pickle.dump(obj, fp)
@@ -39,92 +44,92 @@ def pickle_load(path: str):
         return pickle.load(fp)
 
 
-def f1_with_ci(label_list,
-               pred_list,
-               random_seed: int = 0,
-               n_resamples: int = 1000,
-               confidence_level: List = None,
-               return_ci: bool = False,
-               average='macro'):
-    """ F1 with bootstrap CI (data.shape == (n_sample, 2)) """
-    data = np.array(list(zip(pred_list, label_list)), dtype=object)
-
-    def get_f1(xy, axis=None):
-        assert len(xy.shape) in [2, 3], xy.shape
-        prediction = xy[0]
-        label = xy[1]
-        if axis == -1 and len(xy.shape) == 3:
-            assert average is not None
-            tmp = []
-            for i in tqdm(list(range(len(label)))):
-                tmp.append(f1_score(label[i, :], prediction[i, :], average=average))
-            return np.array(tmp)
-            # return np.array([f1_score(label[i, :], prediction[i, :], average=average)
-            #                  for i in range(len(label))])
-        assert average is not None
-        return f1_score(label, prediction, average=average)
-
-    confidence_level = confidence_level if confidence_level is not None else [90, 95]
-    mean_score = get_f1(data.T)
-    ci = {}
-    if return_ci:
-        for c in confidence_level:
-            logging.debug(f'computing confidence interval: {c}')
-            res = bootstrap((data,),
-                            get_f1,
-                            confidence_level=c * 0.01,
-                            method='percentile',
-                            n_resamples=n_resamples,
-                            random_state=np.random.default_rng(random_seed))
-            ci[str(c)] = [res.confidence_interval.low, res.confidence_interval.high]
-    return mean_score, ci
+# def f1_with_ci(label_list,
+#                pred_list,
+#                random_seed: int = 0,
+#                n_resamples: int = 1000,
+#                confidence_level: List = None,
+#                return_ci: bool = False,
+#                average='macro'):
+#     """ F1 with bootstrap CI (data.shape == (n_sample, 2)) """
+#     data = np.array(list(zip(pred_list, label_list)), dtype=object)
+#
+#     def get_f1(xy, axis=None):
+#         assert len(xy.shape) in [2, 3], xy.shape
+#         prediction = xy[0]
+#         label = xy[1]
+#         if axis == -1 and len(xy.shape) == 3:
+#             assert average is not None
+#             tmp = []
+#             for i in tqdm(list(range(len(label)))):
+#                 tmp.append(f1_score(label[i, :], prediction[i, :], average=average))
+#             return np.array(tmp)
+#             # return np.array([f1_score(label[i, :], prediction[i, :], average=average)
+#             #                  for i in range(len(label))])
+#         assert average is not None
+#         return f1_score(label, prediction, average=average)
+#
+#     confidence_level = confidence_level if confidence_level is not None else [90, 95]
+#     mean_score = get_f1(data.T)
+#     ci = {}
+#     if return_ci:
+#         for c in confidence_level:
+#             logging.debug(f'computing confidence interval: {c}')
+#             res = bootstrap((data,),
+#                             get_f1,
+#                             confidence_level=c * 0.01,
+#                             method='percentile',
+#                             n_resamples=n_resamples,
+#                             random_state=np.random.default_rng(random_seed))
+#             ci[str(c)] = [res.confidence_interval.low, res.confidence_interval.high]
+#     return mean_score, ci
 
 
-def span_f1(pred_list: List,
-            label_list: List,
-            label2id: Dict,
-            span_detection_mode: bool = False,
-            return_ci: bool = False):
-
-    if span_detection_mode:
-        return_ci = False
-
-        def convert_to_binary_mask(entity_label):
-            if entity_label == 'O':
-                return entity_label
-            prefix = entity_label.split('-')[0]  # B or I
-            return '{}-entity'.format(prefix)
-
-        label_list = [[convert_to_binary_mask(_i) for _i in i] for i in label_list]
-        pred_list = [[convert_to_binary_mask(_i) for _i in i] for i in pred_list]
-
-    # compute metrics
-    logging.debug('\n{}'.format(classification_report(label_list, pred_list)))
-    m_micro, ci_micro = f1_with_ci(label_list, pred_list, average='micro', return_ci=return_ci)
-    m_macro, ci_macro = f1_with_ci(label_list, pred_list, average='macro', return_ci=return_ci)
-    metric = {
-        "micro/f1": m_micro,
-        "micro/f1_ci": ci_micro,
-        "micro/recall": recall_score(label_list, pred_list, average='micro'),
-        "micro/precision": precision_score(label_list, pred_list, average='micro'),
-        "macro/f1": m_macro,
-        "macro/f1_ci": ci_macro,
-        "macro/recall": recall_score(label_list, pred_list, average='macro'),
-        "macro/precision": precision_score(label_list, pred_list, average='macro'),
-    }
-    target_names = sorted([k.replace('B-', '') for k in label2id.keys() if k.startswith('B-')])
-    if not span_detection_mode:
-        metric["per_entity_metric"] = {}
-        for t in target_names:
-            _label_list = [[_i if _i.endswith(t) else 'O' for _i in i] for i in label_list]
-            _pred_list = [[_i if _i.endswith(t) else 'O' for _i in i] for i in pred_list]
-            m, ci = f1_with_ci(_label_list, _pred_list, return_ci=return_ci)
-            metric["per_entity_metric"][t] = {
-                "f1": m,
-                "f1_ci": ci,
-                "precision": precision_score(_label_list, _pred_list),
-                "recall": recall_score(_label_list, _pred_list)}
-    return metric
+# def span_f1(pred_list: List,
+#             label_list: List,
+#             label2id: Dict,
+#             span_detection_mode: bool = False,
+#             return_ci: bool = False):
+#
+#     if span_detection_mode:
+#         return_ci = False
+#
+#         def convert_to_binary_mask(entity_label):
+#             if entity_label == 'O':
+#                 return entity_label
+#             prefix = entity_label.split('-')[0]  # B or I
+#             return '{}-entity'.format(prefix)
+#
+#         label_list = [[convert_to_binary_mask(_i) for _i in i] for i in label_list]
+#         pred_list = [[convert_to_binary_mask(_i) for _i in i] for i in pred_list]
+#
+#     # compute metrics
+#     logging.debug('\n{}'.format(classification_report(label_list, pred_list)))
+#     m_micro, ci_micro = f1_with_ci(label_list, pred_list, average='micro', return_ci=return_ci)
+#     m_macro, ci_macro = f1_with_ci(label_list, pred_list, average='macro', return_ci=return_ci)
+#     metric = {
+#         "micro/f1": m_micro,
+#         "micro/f1_ci": ci_micro,
+#         "micro/recall": recall_score(label_list, pred_list, average='micro'),
+#         "micro/precision": precision_score(label_list, pred_list, average='micro'),
+#         "macro/f1": m_macro,
+#         "macro/f1_ci": ci_macro,
+#         "macro/recall": recall_score(label_list, pred_list, average='macro'),
+#         "macro/precision": precision_score(label_list, pred_list, average='macro'),
+#     }
+#     target_names = sorted([k.replace('B-', '') for k in label2id.keys() if k.startswith('B-')])
+#     if not span_detection_mode:
+#         metric["per_entity_metric"] = {}
+#         for t in target_names:
+#             _label_list = [[_i if _i.endswith(t) else 'O' for _i in i] for i in label_list]
+#             _pred_list = [[_i if _i.endswith(t) else 'O' for _i in i] for i in pred_list]
+#             m, ci = f1_with_ci(_label_list, _pred_list, return_ci=return_ci)
+#             metric["per_entity_metric"][t] = {
+#                 "f1": m,
+#                 "f1_ci": ci,
+#                 "precision": precision_score(_label_list, _pred_list),
+#                 "recall": recall_score(_label_list, _pred_list)}
+#     return metric
 
 
 def decode_ner_tags(tag_sequence, input_sequence, probability_sequence=None):
@@ -327,7 +332,7 @@ class NER:
         self.model.to(self.device)
         if self.crf_layer is not None:
             self.crf_layer.to(self.device)
-        logging.debug('{} GPUs are in use'.format(torch.cuda.device_count()))
+        logging.debug(f'{torch.cuda.device_count()} GPUs are in use')
 
         # load pre processor
         if self.crf_layer is not None:
@@ -451,35 +456,36 @@ class NER:
             ind = 0
 
             inputs_list = []
-            for i in loader:
-                if not return_loader:
-                    i = {k: torch.unsqueeze(v, 0) for k, v in i.items()}
-                label = i.pop('labels').cpu().tolist()
-                pred, prob = self.encode_to_prediction(i)
-                assert len(label) == len(pred) == len(prob), str([len(label), len(pred), len(prob)])
-                input_ids = i.pop('input_ids').cpu().tolist()
-                for _i, _p, _prob, _l in zip(input_ids, pred, prob, label):
-                    assert len(_i) == len(_p) == len(_l)
-                    tmp = [(__p, __l, __prob) for __p, __l, __prob in zip(_p, _l, _prob) if __l != PAD_TOKEN_LABEL_ID]
-                    tmp_pred = list(list(zip(*tmp))[0])
-                    tmp_label = list(list(zip(*tmp))[1])
-                    tmp_prob = list(list(zip(*tmp))[2])
-                    if len(tmp_label) != len(labels[ind]):
-                        if len(tmp_label) < len(labels[ind]):
-                            logging.debug('found sequence possibly more than max_length')
-                            logging.debug('{}: \n\t - model loader: {}\n\t - label: {}'.format(ind, tmp_label, labels[ind]))
-                            tmp_pred = tmp_pred + [self.label2id['O']] * (len(labels[ind]) - len(tmp_label))
-                            tmp_prob = tmp_prob + [0.0] * (len(labels[ind]) - len(tmp_label))
-                        else:
-                            raise ValueError(
-                                '{}: \n\t - model loader: {}\n\t - label: {}'.format(ind, tmp_label, labels[ind]))
-                    assert len(tmp_pred) == len(labels[ind])
-                    assert len(inputs[ind]) == len(tmp_pred)
-                    pred_list.append(tmp_pred)
-                    label_list.append(labels[ind])
-                    inputs_list.append(inputs[ind])
-                    prob_list.append(tmp_prob)
-                    ind += 1
+            with torch.no_grad():
+                for i in loader:
+                    if not return_loader:
+                        i = {k: torch.unsqueeze(v, 0) for k, v in i.items()}
+                    label = i.pop('labels').cpu().tolist()
+                    pred, prob = self.encode_to_prediction(i)
+                    assert len(label) == len(pred) == len(prob), str([len(label), len(pred), len(prob)])
+                    input_ids = i.pop('input_ids').cpu().tolist()
+                    for _i, _p, _prob, _l in zip(input_ids, pred, prob, label):
+                        assert len(_i) == len(_p) == len(_l)
+                        tmp = [(__p, __l, __prob) for __p, __l, __prob in zip(_p, _l, _prob) if __l != PAD_TOKEN_LABEL_ID]
+                        tmp_pred = list(list(zip(*tmp))[0])
+                        tmp_label = list(list(zip(*tmp))[1])
+                        tmp_prob = list(list(zip(*tmp))[2])
+                        if len(tmp_label) != len(labels[ind]):
+                            if len(tmp_label) < len(labels[ind]):
+                                logging.debug('found sequence possibly more than max_length')
+                                logging.debug('{}: \n\t - model loader: {}\n\t - label: {}'.format(ind, tmp_label, labels[ind]))
+                                tmp_pred = tmp_pred + [self.label2id['O']] * (len(labels[ind]) - len(tmp_label))
+                                tmp_prob = tmp_prob + [0.0] * (len(labels[ind]) - len(tmp_label))
+                            else:
+                                raise ValueError(
+                                    '{}: \n\t - model loader: {}\n\t - label: {}'.format(ind, tmp_label, labels[ind]))
+                        assert len(tmp_pred) == len(labels[ind])
+                        assert len(inputs[ind]) == len(tmp_pred)
+                        pred_list.append(tmp_pred)
+                        label_list.append(labels[ind])
+                        inputs_list.append(inputs[ind])
+                        prob_list.append(tmp_prob)
+                        ind += 1
             label_list = [[self.id2label[__l] for __l in _l] for _l in label_list]
             pred_list = [[self.id2label[__p] for __p in _p] for _p in pred_list]
             if cache_file_prediction is not None:
